@@ -1,6 +1,8 @@
 #include "adw-demo-page-navigation-view.h"
 
-#include "adw-navigation-view-demo-dialog.h"
+#include <glib/gi18n.h>
+
+#include "adw-navigation-view-demo-window.h"
 
 struct _AdwDemoPageNavigationView
 {
@@ -12,7 +14,11 @@ G_DEFINE_FINAL_TYPE (AdwDemoPageNavigationView, adw_demo_page_navigation_view, A
 static void
 demo_run_cb (AdwDemoPageNavigationView *self)
 {
-  adw_dialog_present (adw_navigation_view_demo_dialog_new (), GTK_WIDGET (self));
+  AdwNavigationViewDemoWindow *window = adw_navigation_view_demo_window_new ();
+  GtkRoot *root = gtk_widget_get_root (GTK_WIDGET (self));
+
+  gtk_window_set_transient_for (GTK_WINDOW (window), GTK_WINDOW (root));
+  gtk_window_present (GTK_WINDOW (window));
 }
 
 static void
