@@ -66,7 +66,7 @@
  * at once, potentially across multiple navigation views.
  *
  * Set [property@HeaderBar:show-back-button] to `FALSE` to disable this behavior
- * in rare scenarios where it's unwanted.
+ * if it's unwanted.
  *
  * `AdwHeaderBar` will also display the title of the `AdwNavigationPage` it's
  * placed into, so most applications shouldn't need to customize it at all.
@@ -227,8 +227,6 @@ typedef struct
   int block_signals;
 
   AdwNavigationView *child_view;
-
-  int nav_split_views;
 } AdwNavigationPagePrivate;
 
 static void adw_navigation_page_buildable_init (GtkBuildableIface *iface);
@@ -267,8 +265,6 @@ struct _AdwNavigationView
   GHashTable *tag_mapping;
   GListStore *navigation_stack;
 
-  gboolean homogeneous[2];
-
   gboolean animate_transitions;
   gboolean pop_on_escape;
 
@@ -279,8 +275,6 @@ struct _AdwNavigationView
   gboolean transition_cancel;
   double transition_progress;
   gboolean gesture_active;
-  /* AdwNavigationDirection or -1 */
-  int swipe_direction;
 
   AdwShadowHelper *shadow_helper;
   AdwSwipeTracker *swipe_tracker;
@@ -300,9 +294,6 @@ G_DEFINE_FINAL_TYPE_WITH_CODE (AdwNavigationView, adw_navigation_view, GTK_TYPE_
 enum {
   PROP_0,
   PROP_VISIBLE_PAGE,
-  PROP_VISIBLE_PAGE_TAG,
-  PROP_HHOMOGENEOUS,
-  PROP_VHOMOGENEOUS,
   PROP_ANIMATE_TRANSITIONS,
   PROP_POP_ON_ESCAPE,
   PROP_NAVIGATION_STACK,
@@ -342,9 +333,6 @@ adw_navigation_view_model_get_n_items (GListModel *model)
 {
   AdwNavigationViewModel *self = ADW_NAVIGATION_VIEW_MODEL (model);
 
-  if (G_UNLIKELY (!ADW_IS_NAVIGATION_VIEW (self->view)))
-    return 0;
-
   return g_list_model_get_n_items (G_LIST_MODEL (self->view->navigation_stack));
 }
 
@@ -354,9 +342,6 @@ adw_navigation_view_model_get_item (GListModel *model,
 {
   AdwNavigationViewModel *self = ADW_NAVIGATION_VIEW_MODEL (model);
 
-  if (G_UNLIKELY (!ADW_IS_NAVIGATION_VIEW (self->view)))
-    return NULL;
-
   return g_list_model_get_item (G_LIST_MODEL (self->view->navigation_stack),
                                 position);
 }
@@ -372,27 +357,14 @@ adw_navigation_view_model_list_model_init (GListModelInterface *iface)
 G_DEFINE_FINAL_TYPE_WITH_CODE (AdwNavigationViewModel, adw_navigation_view_model, G_TYPE_OBJECT,
                                G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL, adw_navigation_view_model_list_model_init))
 
-static void
-adw_navigation_view_model_dispose (GObject *object)
-{
-  AdwNavigationViewModel *self = ADW_NAVIGATION_VIEW_MODEL (object);
-
-  g_clear_weak_pointer (&self->view);
-
-  G_OBJECT_CLASS (adw_navigation_view_model_parent_class)->dispose (object);
-}
-
 static void
 adw_navigation_view_model_init (AdwNavigationViewModel *self)
 {
 }
 
 static void
-adw_navigation_view_model_class_init (AdwNavigationViewModelClass *klass)
+adw_navigation_view_model_class_init (AdwNavigationViewModelClass *class)
 {
-  GObjectClass *object_class = G_OBJECT_CLASS (klass);
-
-  object_class->dispose = adw_navigation_view_model_dispose;
 }
 
 static GListModel *
@@ -401,7 +373,7 @@ adw_navigation_view_model_new (AdwNavigationView *view)
   AdwNavigationViewModel *model;
 
   model = g_object_new (ADW_TYPE_NAVIGATION_VIEW_MODEL, NULL);
-  g_set_weak_pointer (&model->view, view);
+  model->view = view;
 
   return G_LIST_MODEL (model);
 }
@@ -431,7 +403,7 @@ adw_navigation_page_realize (GtkWidget *widget)
 
   GTK_WIDGET_CLASS (adw_navigation_page_parent_class)->realize (widget);
 
-  if (!(priv->title && *priv->title) && !priv->child_view && priv->nav_split_views == 0) {
+  if (!(priv->title && *priv->title) && !priv->child_view) {
     g_warning ("AdwNavigationPage %p is missing a title. To hide a header bar " \
                "title, consider using AdwHeaderBar:show-title instead.", self);
   }
@@ -444,7 +416,13 @@ adw_navigation_page_dispose (GObject *object)
   AdwNavigationPagePrivate *priv = adw_navigation_page_get_instance_private (self);
 
   g_clear_pointer (&priv->child, gtk_widget_unparent);
-  g_clear_weak_pointer (&priv->child_view);
+
+  if (priv->child_view) {
+    g_object_remove_weak_pointer (G_OBJECT (priv->child_view),
+                                  (gpointer *) &priv->child_view);
+
+    priv->child_view = NULL;
+  }
 
   G_OBJECT_CLASS (adw_navigation_page_parent_class)->dispose (object);
 }
@@ -457,7 +435,10 @@ adw_navigation_page_finalize (GObject *object)
 
   g_free (priv->title);
   g_free (priv->tag);
-  g_clear_weak_pointer (&priv->last_focus);
+
+  if (priv->last_focus)
+    g_object_remove_weak_pointer (G_OBJECT (priv->last_focus),
+                                  (gpointer *) &priv->last_focus);
 
   G_OBJECT_CLASS (adw_navigation_page_parent_class)->finalize (object);
 }
@@ -514,26 +495,6 @@ adw_navigation_page_set_property (GObject      *object,
   }
 }
 
-static void
-adw_navigation_page_real_showing (AdwNavigationPage *self)
-{
-}
-
-static void
-adw_navigation_page_real_shown (AdwNavigationPage *self)
-{
-}
-
-static void
-adw_navigation_page_real_hiding (AdwNavigationPage *self)
-{
-}
-
-static void
-adw_navigation_page_real_hidden (AdwNavigationPage *self)
-{
-}
-
 static void
 adw_navigation_page_class_init (AdwNavigationPageClass *klass)
 {
@@ -548,13 +509,8 @@ adw_navigation_page_class_init (AdwNavigationPageClass *klass)
   widget_class->realize = adw_navigation_page_realize;
   widget_class->compute_expand = adw_widget_compute_expand;
 
-  klass->showing = adw_navigation_page_real_showing;
-  klass->shown = adw_navigation_page_real_shown;
-  klass->hiding = adw_navigation_page_real_hiding;
-  klass->hidden = adw_navigation_page_real_hidden;
-
   /**
-   * AdwNavigationPage:child:
+   * AdwNavigationPage:child: (attributes org.gtk.Property.get=adw_navigation_page_get_child org.gtk.Property.set=adw_navigation_page_set_child)
    *
    * The child widget.
    *
@@ -566,7 +522,7 @@ adw_navigation_page_class_init (AdwNavigationPageClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwNavigationPage:tag:
+   * AdwNavigationPage:tag: (attributes org.gtk.Property.get=adw_navigation_page_get_tag org.gtk.Property.set=adw_navigation_page_set_tag)
    *
    * The page tag.
    *
@@ -587,7 +543,7 @@ adw_navigation_page_class_init (AdwNavigationPageClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwNavigationPage:title:
+   * AdwNavigationPage:title: (attributes org.gtk.Property.get=adw_navigation_page_get_title org.gtk.Property.set=adw_navigation_page_set_title)
    *
    * The page title.
    *
@@ -602,7 +558,7 @@ adw_navigation_page_class_init (AdwNavigationPageClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwNavigationPage:can-pop:
+   * AdwNavigationPage:can-pop: (attributes org.gtk.Property.get=adw_navigation_page_get_can_pop org.gtk.Property.set=adw_navigation_page_set_can_pop)
    *
    * Whether the page can be popped from navigation stack.
    *
@@ -788,7 +744,12 @@ switch_page (AdwNavigationView *self,
 
     contains_focus = TRUE;
 
-    g_set_weak_pointer (&priv->last_focus, focus);
+    if (priv->last_focus)
+      g_object_remove_weak_pointer (G_OBJECT (priv->last_focus),
+                                    (gpointer *)&priv->last_focus);
+    priv->last_focus = focus;
+    g_object_add_weak_pointer (G_OBJECT (priv->last_focus),
+                               (gpointer *)&priv->last_focus);
   }
 
   if (!prev_page)
@@ -857,11 +818,6 @@ switch_page (AdwNavigationView *self,
     adw_animation_skip (self->transition);
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_VISIBLE_PAGE]);
-
-  if ((prev_page && adw_navigation_page_get_tag (prev_page)) ||
-      (page && adw_navigation_page_get_tag (page))) {
-    g_object_notify_by_pspec (G_OBJECT (self), props[PROP_VISIBLE_PAGE_TAG]);
-  }
 }
 
 static void
@@ -912,9 +868,6 @@ pop_from_stack (AdwNavigationView *self,
 
   old_page = adw_navigation_view_get_visible_page (self);
 
-  if (page_to == old_page)
-    return;
-
   length = g_list_model_get_n_items (G_LIST_MODEL (self->navigation_stack));
 
   g_assert (g_list_store_find (self->navigation_stack, page_to, &pos));
@@ -1268,41 +1221,22 @@ static void
 prepare_cb (AdwSwipeTracker        *tracker,
             AdwNavigationDirection  direction,
             AdwNavigationView      *self)
-{
-  if (!adw_navigation_view_get_visible_page (self))
-    return;
-
-  self->swipe_direction = ADW_NAVIGATION_DIRECTION_BACK;
-}
-
-static void
-begin_swipe_cb (AdwSwipeTracker   *tracker,
-                AdwNavigationView *self)
 {
   AdwNavigationPage *visible_page = adw_navigation_view_get_visible_page (self);
   AdwNavigationPage *new_page;
   gboolean remove_on_pop = FALSE;
 
-  if (self->swipe_direction < 0)
+  if (!visible_page)
     return;
 
-  if (!visible_page) {
-    self->swipe_direction = -1;
-    return;
-  }
-
-  if (self->swipe_direction == ADW_NAVIGATION_DIRECTION_BACK) {
-    if (!adw_navigation_page_get_can_pop (visible_page)) {
-      self->swipe_direction = -1;
+  if (direction == ADW_NAVIGATION_DIRECTION_BACK) {
+    if (!adw_navigation_page_get_can_pop (visible_page))
       return;
-    }
 
     new_page = adw_navigation_view_get_previous_page (self, visible_page);
 
-    if (!new_page) {
-      self->swipe_direction = -1;
+    if (!new_page)
       return;
-    }
 
   } else {
     new_page = get_next_page (self);
@@ -1320,9 +1254,9 @@ begin_swipe_cb (AdwSwipeTracker   *tracker,
   self->showing_page = new_page;
   self->hiding_page = g_object_ref (visible_page);
 
-  self->transition_pop = (self->swipe_direction == ADW_NAVIGATION_DIRECTION_BACK);
+  self->transition_pop = (direction == ADW_NAVIGATION_DIRECTION_BACK);
 
-  if (self->swipe_direction == ADW_NAVIGATION_DIRECTION_BACK) {
+  if (direction == ADW_NAVIGATION_DIRECTION_BACK) {
     g_object_ref (new_page);
   } else {
     if (remove_on_pop)
@@ -1347,8 +1281,6 @@ begin_swipe_cb (AdwSwipeTracker   *tracker,
   gtk_widget_queue_resize (GTK_WIDGET (self));
 
   adw_swipe_tracker_set_upper_overshoot (self->swipe_tracker, TRUE);
-
-  self->swipe_direction = -1;
 }
 
 static void
@@ -1375,8 +1307,6 @@ end_swipe_cb (AdwSwipeTracker   *tracker,
 {
   gboolean animate;
 
-  self->swipe_direction = -1;
-
   if (!self->gesture_active)
     return;
 
@@ -1425,7 +1355,20 @@ set_child_view (AdwNavigationPage *self,
 {
   AdwNavigationPagePrivate *priv = adw_navigation_page_get_instance_private (self);
 
-  g_set_weak_pointer (&priv->child_view, view);
+  if (view == priv->child_view)
+    return;
+
+  if (priv->child_view) {
+    g_object_remove_weak_pointer (G_OBJECT (priv->child_view),
+                                  (gpointer *) &priv->child_view);
+  }
+
+  priv->child_view = view;
+
+  if (priv->child_view) {
+    g_object_add_weak_pointer (G_OBJECT (priv->child_view),
+                               (gpointer *) &priv->child_view);
+  }
 }
 
 static void
@@ -1438,51 +1381,29 @@ adw_navigation_view_measure (GtkWidget      *widget,
                              int            *natural_baseline)
 {
   AdwNavigationView *self = ADW_NAVIGATION_VIEW (widget);
-  int min = 0, nat = 0;
+  AdwNavigationPage *visible_page = NULL;
+  int min = 0, nat = 0, min_baseline = -1, nat_baseline = -1;
+  int last_min = 0, last_nat = 0, last_min_baseline = -1, last_nat_baseline = -1;
 
-  if (self->homogeneous[orientation]) {
-    int child_min = 0, child_nat = 0;
-    GtkWidget *child;
+  visible_page = adw_navigation_view_get_visible_page (self);
 
-    for (child = gtk_widget_get_first_child (GTK_WIDGET (self));
-         child;
-         child = gtk_widget_get_next_sibling (child)) {
-      if (!ADW_IS_NAVIGATION_PAGE (child))
-        continue;
+  if (visible_page)
+    gtk_widget_measure (GTK_WIDGET (visible_page), orientation, for_size,
+                        &min, &nat, &min_baseline, &nat_baseline);
 
-      gtk_widget_measure (child, orientation, for_size,
-                          &child_min, &child_nat, NULL, NULL);
-
-      min = MAX (min, child_min);
-      nat = MAX (nat, child_nat);
-    }
-  } else {
-    AdwNavigationPage *visible_page = adw_navigation_view_get_visible_page (self);
-
-    if (visible_page) {
-      gtk_widget_measure (GTK_WIDGET (visible_page), orientation, for_size,
-                          &min, &nat, NULL, NULL);
-    }
-
-    if (self->hiding_page) {
-      int last_min = 0, last_nat = 0;
-
-      gtk_widget_measure (GTK_WIDGET (self->hiding_page), orientation, for_size,
-                          &last_min, &last_nat, NULL, NULL);
-
-      min = MAX (min, last_min);
-      nat = MAX (nat, last_nat);
-    }
-  }
+  if (self->hiding_page)
+    gtk_widget_measure (GTK_WIDGET (self->hiding_page),
+                        orientation, for_size, &last_min, &last_nat,
+                        &last_min_baseline, &last_nat_baseline);
 
   if (minimum)
-    *minimum = min;
+    *minimum = MAX (min, last_min);
   if (natural)
-    *natural = nat;
+    *natural = MAX (nat, last_nat);
   if (minimum_baseline)
-    *minimum_baseline = -1;
+    *minimum_baseline = MAX (min_baseline, last_min_baseline);
   if (natural_baseline)
-    *natural_baseline = -1;
+    *natural_baseline = MAX (nat_baseline, last_nat_baseline);
 }
 
 static void
@@ -1650,7 +1571,7 @@ adw_navigation_view_root (GtkWidget *widget)
 
   GTK_WIDGET_CLASS (adw_navigation_view_parent_class)->root (widget);
 
-  parent_page = adw_widget_get_ancestor (widget, ADW_TYPE_NAVIGATION_PAGE, TRUE, TRUE);
+  parent_page = adw_widget_get_ancestor_same_native (widget, ADW_TYPE_NAVIGATION_PAGE);
 
   if (parent_page)
     set_child_view (ADW_NAVIGATION_PAGE (parent_page), self);
@@ -1661,7 +1582,7 @@ adw_navigation_view_unroot (GtkWidget *widget)
 {
   GtkWidget *parent_page;
 
-  parent_page = adw_widget_get_ancestor (widget, ADW_TYPE_NAVIGATION_PAGE, TRUE, TRUE);
+  parent_page = adw_widget_get_ancestor_same_native (widget, ADW_TYPE_NAVIGATION_PAGE);
 
   if (parent_page)
     set_child_view (ADW_NAVIGATION_PAGE (parent_page), NULL);
@@ -1709,7 +1630,9 @@ adw_navigation_view_finalize (GObject *object)
 {
   AdwNavigationView *self = ADW_NAVIGATION_VIEW (object);
 
-  g_clear_weak_pointer (&self->navigation_stack_model);
+  if (self->navigation_stack_model)
+    g_object_remove_weak_pointer (G_OBJECT (self->navigation_stack_model),
+                                  (gpointer *) &self->navigation_stack_model);
 
   G_OBJECT_CLASS (adw_navigation_view_parent_class)->finalize (object);
 }
@@ -1726,15 +1649,6 @@ adw_navigation_view_get_property (GObject    *object,
   case PROP_VISIBLE_PAGE:
     g_value_set_object (value, adw_navigation_view_get_visible_page (self));
     break;
-  case PROP_VISIBLE_PAGE_TAG:
-    g_value_set_string (value, adw_navigation_view_get_visible_page_tag (self));
-    break;
-  case PROP_HHOMOGENEOUS:
-    g_value_set_boolean (value, adw_navigation_view_get_hhomogeneous (self));
-    break;
-  case PROP_VHOMOGENEOUS:
-    g_value_set_boolean (value, adw_navigation_view_get_vhomogeneous (self));
-    break;
   case PROP_ANIMATE_TRANSITIONS:
     g_value_set_boolean (value, adw_navigation_view_get_animate_transitions (self));
     break;
@@ -1758,12 +1672,6 @@ adw_navigation_view_set_property (GObject      *object,
   AdwNavigationView *self = ADW_NAVIGATION_VIEW (object);
 
   switch (prop_id) {
-  case PROP_HHOMOGENEOUS:
-    adw_navigation_view_set_hhomogeneous (self, g_value_get_boolean (value));
-    break;
-  case PROP_VHOMOGENEOUS:
-    adw_navigation_view_set_vhomogeneous (self, g_value_get_boolean (value));
-    break;
   case PROP_ANIMATE_TRANSITIONS:
     adw_navigation_view_set_animate_transitions (self, g_value_get_boolean (value));
     break;
@@ -1809,7 +1717,7 @@ adw_navigation_view_class_init (AdwNavigationViewClass *klass)
   widget_class->compute_expand = adw_widget_compute_expand;
 
   /**
-   * AdwNavigationView:visible-page:
+   * AdwNavigationView:visible-page: (attributes org.gtk.Property.get=adw_navigation_view_get_visible_page)
    *
    * The currently visible page.
    *
@@ -1821,55 +1729,7 @@ adw_navigation_view_class_init (AdwNavigationViewClass *klass)
                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * AdwNavigationView:visible-page-tag:
-   *
-   * The tag of the currently visible page.
-   *
-   * Since: 1.7
-   */
-  props[PROP_VISIBLE_PAGE_TAG] =
-    g_param_spec_string ("visible-page-tag", NULL, NULL,
-                         NULL,
-                         G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
-
-  /**
-   * AdwNavigationView:hhomogeneous:
-   *
-   * Whether the view is horizontally homogeneous.
-   *
-   * If the view is horizontally homogeneous, it allocates the same width for
-   * all pages.
-   *
-   * If it's not, the page may change width when a different page becomes
-   * visible.
-   *
-   * Since: 1.7
-   */
-  props[PROP_HHOMOGENEOUS] =
-    g_param_spec_boolean ("hhomogeneous", NULL, NULL,
-                          FALSE,
-                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
-
-  /**
-   * AdwNavigationView:vhomogeneous:
-   *
-   * Whether the view is vertically homogeneous.
-   *
-   * If the view is vertically homogeneous, it allocates the same height for
-   * all pages.
-   *
-   * If it's not, the view may change height when a different page becomes
-   * visible.
-   *
-   * Since: 1.7
-   */
-  props[PROP_VHOMOGENEOUS] =
-    g_param_spec_boolean ("vhomogeneous", NULL, NULL,
-                          FALSE,
-                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
-
-  /**
-   * AdwNavigationView:animate-transitions:
+   * AdwNavigationView:animate-transitions: (attributes org.gtk.Property.get=adw_navigation_view_get_animate_transitions org.gtk.Property.set=adw_navigation_view_set_animate_transitions)
    *
    * Whether to animate page transitions.
    *
@@ -1883,7 +1743,7 @@ adw_navigation_view_class_init (AdwNavigationViewClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwNavigationView:pop-on-escape:
+   * AdwNavigationView:pop-on-escape: (attributes org.gtk.Property.get=adw_navigation_view_get_pop_on_escape org.gtk.Property.set=adw_navigation_view_set_pop_on_escape)
    *
    * Whether pressing Escape pops the current page.
    *
@@ -1898,7 +1758,7 @@ adw_navigation_view_class_init (AdwNavigationViewClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwNavigationView:navigation-stack:
+   * AdwNavigationView:navigation-stack: (attributes org.gtk.Property.get=adw_navigation_view_get_navigation_stack)
    *
    * A list model that contains the pages in navigation stack.
    *
@@ -2078,15 +1938,11 @@ adw_navigation_view_init (AdwNavigationView *self)
 
   g_signal_connect (self->swipe_tracker, "prepare",
                     G_CALLBACK (prepare_cb), self);
-  g_signal_connect (self->swipe_tracker, "begin-swipe",
-                    G_CALLBACK (begin_swipe_cb), self);
   g_signal_connect (self->swipe_tracker, "update-swipe",
                     G_CALLBACK (update_swipe_cb), self);
   g_signal_connect (self->swipe_tracker, "end-swipe",
                     G_CALLBACK (end_swipe_cb), self);
 
-  self->swipe_direction = -1;
-
   self->shield = adw_gizmo_new ("widget", NULL, NULL, NULL, NULL, NULL, NULL);
   gtk_widget_set_child_visible (self->shield, FALSE);
   gtk_widget_set_parent (self->shield, GTK_WIDGET (self));
@@ -2239,7 +2095,7 @@ adw_navigation_page_new_with_tag (GtkWidget  *child,
 }
 
 /**
- * adw_navigation_page_get_child:
+ * adw_navigation_page_get_child: (attributes org.gtk.Method.get_property=child)
  * @self: a navigation page
  *
  * Gets the child widget of @self.
@@ -2261,7 +2117,7 @@ adw_navigation_page_get_child (AdwNavigationPage *self)
 }
 
 /**
- * adw_navigation_page_set_child:
+ * adw_navigation_page_set_child: (attributes org.gtk.Method.set_property=child)
  * @self: a navigation page
  * @child: (nullable): the child widget
  *
@@ -2278,14 +2134,14 @@ adw_navigation_page_set_child (AdwNavigationPage *self,
   g_return_if_fail (ADW_IS_NAVIGATION_PAGE (self));
   g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
 
+  if (child)
+    g_return_if_fail (gtk_widget_get_parent (child) == NULL);
+
   priv = adw_navigation_page_get_instance_private (self);
 
   if (priv->child == child)
     return;
 
-  if (child)
-    g_return_if_fail (gtk_widget_get_parent (child) == NULL);
-
   g_object_freeze_notify (G_OBJECT (self));
 
   if (priv->child)
@@ -2302,7 +2158,7 @@ adw_navigation_page_set_child (AdwNavigationPage *self,
 }
 
 /**
- * adw_navigation_page_get_tag:
+ * adw_navigation_page_get_tag: (attributes org.gtk.Method.get_property=tag)
  * @self: a navigation page
  *
  * Gets the tag of @self.
@@ -2324,7 +2180,7 @@ adw_navigation_page_get_tag (AdwNavigationPage *self)
 }
 
 /**
- * adw_navigation_page_set_tag:
+ * adw_navigation_page_set_tag: (attributes org.gtk.Method.set_property=tag)
  * @self: a navigation page
  * @tag: (nullable): the page tag
  *
@@ -2380,7 +2236,7 @@ adw_navigation_page_set_tag (AdwNavigationPage *self,
 }
 
 /**
- * adw_navigation_page_get_title:
+ * adw_navigation_page_get_title: (attributes org.gtk.Method.get_property=title)
  * @self: a navigation page
  *
  * Gets the title of @self.
@@ -2402,7 +2258,7 @@ adw_navigation_page_get_title (AdwNavigationPage *self)
 }
 
 /**
- * adw_navigation_page_set_title:
+ * adw_navigation_page_set_title: (attributes org.gtk.Method.set_property=title)
  * @self: a navigation page
  * @title: the title
  *
@@ -2435,7 +2291,7 @@ adw_navigation_page_set_title (AdwNavigationPage *self,
 }
 
 /**
- * adw_navigation_page_get_can_pop:
+ * adw_navigation_page_get_can_pop: (attributes org.gtk.Method.get_property=can-pop)
  * @self: a navigation page
  *
  * Gets whether @self can be popped from navigation stack.
@@ -2457,7 +2313,7 @@ adw_navigation_page_get_can_pop (AdwNavigationPage *self)
 }
 
 /**
- * adw_navigation_page_set_can_pop:
+ * adw_navigation_page_set_can_pop: (attributes org.gtk.Method.set_property=can-pop)
  * @self: a navigation page
  * @can_pop: whether the page can be popped from navigation stack
  *
@@ -2572,26 +2428,6 @@ adw_navigation_page_unblock_signals (AdwNavigationPage *self)
   priv->block_signals--;
 }
 
-void
-adw_navigation_page_add_child_nav_split_view (AdwNavigationPage *self)
-{
-  AdwNavigationPagePrivate *priv = adw_navigation_page_get_instance_private (self);
-
-  g_return_if_fail (ADW_IS_NAVIGATION_PAGE (self));
-
-  priv->nav_split_views++;
-}
-
-void
-adw_navigation_page_remove_child_nav_split_view (AdwNavigationPage *self)
-{
-  AdwNavigationPagePrivate *priv = adw_navigation_page_get_instance_private (self);
-
-  g_return_if_fail (ADW_IS_NAVIGATION_PAGE (self));
-
-  priv->nav_split_views--;
-}
-
 /**
  * adw_navigation_view_new:
  *
@@ -2699,7 +2535,7 @@ adw_navigation_view_find_page (AdwNavigationView *self,
  * If [method@NavigationView.add] hasn't been called, the page is automatically
  * removed once it's popped.
  *
- * [signal@NavigationView::pushed] will be emitted for @page.
+ * [signal@NavigationView::popped] will be emitted for @page.
  *
  * See [method@NavigationView.push_by_tag].
  *
@@ -2728,7 +2564,7 @@ adw_navigation_view_push (AdwNavigationView *self,
  * If [method@NavigationView.add] hasn't been called, the page is automatically
  * removed once it's popped.
  *
- * [signal@NavigationView::pushed] will be emitted for the page.
+ * [signal@NavigationView::popped] will be emitted for pushed page.
  *
  * See [method@NavigationView.push] and [property@NavigationPage:tag].
  *
@@ -2912,16 +2748,15 @@ adw_navigation_view_replace (AdwNavigationView  *self,
                              AdwNavigationPage **pages,
                              int                 n_pages)
 {
-  AdwNavigationPage *visible_page;
+  AdwNavigationPage *visible_page, *old_visible_page;
   GHashTable *added_pages;
   guint i, old_length;
-  gboolean had_visible_page, old_visible_page_had_tag = FALSE;
 
   g_return_if_fail (ADW_IS_NAVIGATION_VIEW (self));
   g_return_if_fail (n_pages >= 0);
 
   visible_page = adw_navigation_view_get_visible_page (self);
-  had_visible_page = visible_page != NULL;
+  old_visible_page = visible_page;
   old_length = g_list_model_get_n_items (G_LIST_MODEL (self->navigation_stack));
 
   added_pages = g_hash_table_new (g_direct_hash, g_direct_equal);
@@ -2942,7 +2777,6 @@ adw_navigation_view_replace (AdwNavigationView  *self,
     if (get_remove_on_pop (c) &&
         !g_hash_table_contains (added_pages, c)) {
       if (c == visible_page) {
-        old_visible_page_had_tag = adw_navigation_page_get_tag (visible_page) != NULL;
         adw_navigation_page_hiding (visible_page);
         adw_navigation_page_hidden (visible_page);
         visible_page = NULL;
@@ -2984,11 +2818,8 @@ adw_navigation_view_replace (AdwNavigationView  *self,
       switch_page (self, visible_page, new_visible_page, TRUE, FALSE, 0);
   } else if (visible_page) {
     switch_page (self, visible_page, NULL, TRUE, FALSE, 0);
-  } else if (had_visible_page) {
+  } else if (old_visible_page) {
     g_object_notify_by_pspec (G_OBJECT (self), props[PROP_VISIBLE_PAGE]);
-
-    if (old_visible_page_had_tag)
-      g_object_notify_by_pspec (G_OBJECT (self), props[PROP_VISIBLE_PAGE_TAG]);
   }
 
   g_hash_table_unref (added_pages);
@@ -3060,7 +2891,7 @@ adw_navigation_view_replace_with_tags (AdwNavigationView  *self,
 }
 
 /**
- * adw_navigation_view_get_visible_page:
+ * adw_navigation_view_get_visible_page: (attributes org.gtk.Method.get_property=visible-page)
  * @self: a navigation view
  *
  * Gets the currently visible page in @self.
@@ -3089,33 +2920,6 @@ adw_navigation_view_get_visible_page (AdwNavigationView *self)
   return ret;
 }
 
-/**
- * adw_navigation_view_get_visible_page_tag:
- * @self: a navigation view
- *
- * Gets the tag of the currently visible page in @self.
- *
- * Returns: (transfer none) (nullable): the tag of the currently visible page
- *
- * Since: 1.7
- */
-const char *
-adw_navigation_view_get_visible_page_tag (AdwNavigationView *self)
-{
-  g_return_val_if_fail (ADW_IS_NAVIGATION_VIEW (self), NULL);
-
-  AdwNavigationPage *current_page;
-
-  g_return_val_if_fail (ADW_IS_NAVIGATION_VIEW (self), NULL);
-
-  current_page = adw_navigation_view_get_visible_page (self);
-
-  if (current_page == NULL)
-    return NULL;
-
-  return adw_navigation_page_get_tag (current_page);
-}
-
 /**
  * adw_navigation_view_get_previous_page:
  * @self: a navigation view
@@ -3156,110 +2960,7 @@ adw_navigation_view_get_previous_page (AdwNavigationView *self,
 }
 
 /**
- * adw_navigation_view_get_hhomogeneous:
- * @self: a navigation view
- *
- * Gets whether @self is horizontally homogeneous.
- *
- * Returns: whether @self is horizontally homogeneous
- *
- * Since: 1.7
- */
-gboolean
-adw_navigation_view_get_hhomogeneous (AdwNavigationView *self)
-{
-  g_return_val_if_fail (ADW_IS_NAVIGATION_VIEW (self), FALSE);
-
-  return self->homogeneous[GTK_ORIENTATION_HORIZONTAL];
-}
-
-/**
- * adw_navigation_view_set_hhomogeneous:
- * @self: a navigation view
- * @hhomogeneous: whether to make @self horizontally homogeneous
- *
- * Sets @self to be horizontally homogeneous or not.
- *
- * If the view is horizontally homogeneous, it allocates the same width for
- * all pages.
- *
- * If it's not, the view may change width when a different page becomes visible.
- *
- * Since: 1.7
- */
-void
-adw_navigation_view_set_hhomogeneous (AdwNavigationView *self,
-                                      gboolean           hhomogeneous)
-{
-  g_return_if_fail (ADW_IS_NAVIGATION_VIEW (self));
-
-  hhomogeneous = !!hhomogeneous;
-
-  if (self->homogeneous[GTK_ORIENTATION_HORIZONTAL] == hhomogeneous)
-    return;
-
-  self->homogeneous[GTK_ORIENTATION_HORIZONTAL] = hhomogeneous;
-
-  if (gtk_widget_get_visible (GTK_WIDGET (self)))
-    gtk_widget_queue_resize (GTK_WIDGET (self));
-
-  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_HHOMOGENEOUS]);
-}
-
-/**
- * adw_navigation_view_get_vhomogeneous:
- * @self: a navigation view
- *
- * Gets whether @self is vertically homogeneous.
- *
- * Returns: whether @self is vertically homogeneous
- *
- * Since: 1.7
- */
-gboolean
-adw_navigation_view_get_vhomogeneous (AdwNavigationView *self)
-{
-  g_return_val_if_fail (ADW_IS_NAVIGATION_VIEW (self), FALSE);
-
-  return self->homogeneous[GTK_ORIENTATION_VERTICAL];
-}
-
-/**
- * adw_navigation_view_set_vhomogeneous:
- * @self: a navigation view
- * @vhomogeneous: whether to make @self vertically homogeneous
- *
- * Sets @self to be vertically homogeneous or not.
- *
- * If the view is vertically homogeneous, it allocates the same height for
- * all pages.
- *
- * If it's not, the view may change height when a different page becomes
- * visible.
- *
- * Since: 1.7
- */
-void
-adw_navigation_view_set_vhomogeneous (AdwNavigationView *self,
-                                      gboolean           vhomogeneous)
-{
-  g_return_if_fail (ADW_IS_NAVIGATION_VIEW (self));
-
-  vhomogeneous = !!vhomogeneous;
-
-  if (self->homogeneous[GTK_ORIENTATION_VERTICAL] == vhomogeneous)
-    return;
-
-  self->homogeneous[GTK_ORIENTATION_VERTICAL] = vhomogeneous;
-
-  if (gtk_widget_get_visible (GTK_WIDGET (self)))
-    gtk_widget_queue_resize (GTK_WIDGET (self));
-
-  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_VHOMOGENEOUS]);
-}
-
-/**
- * adw_navigation_view_get_animate_transitions:
+ * adw_navigation_view_get_animate_transitions: (attributes org.gtk.Method.get_property=animate-transitions)
  * @self: a navigation view
  *
  * Gets whether @self animates page transitions.
@@ -3277,7 +2978,7 @@ adw_navigation_view_get_animate_transitions (AdwNavigationView *self)
 }
 
 /**
- * adw_navigation_view_set_animate_transitions:
+ * adw_navigation_view_set_animate_transitions: (attributes org.gtk.Method.set_property=animate-transitions)
  * @self: a navigation view
  * @animate_transitions: whether to animate page transitions
  *
@@ -3304,7 +3005,7 @@ adw_navigation_view_set_animate_transitions (AdwNavigationView *self,
 }
 
 /**
- * adw_navigation_view_get_pop_on_escape:
+ * adw_navigation_view_get_pop_on_escape: (attributes org.gtk.Method.get_property=pop-on-escape)
  * @self: a navigation view
  *
  * Gets whether pressing Escape pops the current page on @self.
@@ -3322,7 +3023,7 @@ adw_navigation_view_get_pop_on_escape (AdwNavigationView *self)
 }
 
 /**
- * adw_navigation_view_set_pop_on_escape:
+ * adw_navigation_view_set_pop_on_escape: (attributes org.gtk.Method.set_property=pop-on-escape)
  * @self: a navigation view
  * @pop_on_escape: whether to pop the current page when pressing Escape
  *
@@ -3350,7 +3051,7 @@ adw_navigation_view_set_pop_on_escape (AdwNavigationView *self,
 }
 
 /**
- * adw_navigation_view_get_navigation_stack:
+ * adw_navigation_view_get_navigation_stack: (attributes org.gtk.Method.get_property=navigation-stack)
  * @self: a navigation view
  *
  * Returns a [iface@Gio.ListModel] that contains the pages in navigation stack.
@@ -3371,8 +3072,9 @@ adw_navigation_view_get_navigation_stack (AdwNavigationView *self)
   if (self->navigation_stack_model)
     return g_object_ref (self->navigation_stack_model);
 
-  g_set_weak_pointer (&self->navigation_stack_model,
-                      adw_navigation_view_model_new (self));
+  self->navigation_stack_model = adw_navigation_view_model_new (self);
+  g_object_add_weak_pointer (G_OBJECT (self->navigation_stack_model),
+                             (gpointer *) &self->navigation_stack_model);
 
   return self->navigation_stack_model;
 }
