@@ -52,8 +52,8 @@
  *
  * By default, top and bottom bars are flat and scrolling content has a subtle
  * undershoot shadow, same as when using the
- * [`.undershoot-top`](style-classes.html#undershoot-indicators) and
- * [`.undershoot-bottom`](style-classes.html#undershoot-indicators) style
+ * [`.undershoot-top`](style-classes.html#undershot-indicators) and
+ * [`.undershoot-bottom`](style-classes.html#undershot-indicators) style
  * classes. This works well in most cases, e.g. with [class@StatusPage] or
  * [class@PreferencesPage], where the background at the top and bottom parts of
  * the page is uniform. Additionally, windows with sidebars should always use
@@ -283,40 +283,9 @@ adw_toolbar_view_measure (GtkWidget      *widget,
   gtk_widget_measure (self->bottom_bar, orientation, -1,
                       &bottom_min, &bottom_nat, NULL, NULL);
 
-  if (self->content) {
-    if (for_size < 0 || orientation == GTK_ORIENTATION_VERTICAL ||
-        (self->extend_content_to_top_edge && self->extend_content_to_bottom_edge)) {
-      gtk_widget_measure (self->content, orientation, for_size,
-                          &content_min, &content_nat, NULL, NULL);
-    } else {
-      int top_min_height, bottom_min_height;
-      int top_nat_height, bottom_nat_height;
-      int for_size_min, for_size_nat;
-
-      gtk_widget_measure (self->top_bar, GTK_ORIENTATION_VERTICAL, -1,
-                          &top_min_height, &top_nat_height, NULL, NULL);
-      gtk_widget_measure (self->bottom_bar, GTK_ORIENTATION_VERTICAL, -1,
-                          &bottom_min_height, &bottom_nat_height, NULL, NULL);
-
-      for_size_min = for_size_nat = for_size;
-      if (!self->extend_content_to_top_edge) {
-        for_size_min -= top_min_height;
-        for_size_nat -= top_nat_height;
-      }
-      if (!self->extend_content_to_bottom_edge) {
-        for_size_min -= bottom_min_height;
-        for_size_nat -= bottom_nat_height;
-      }
-
-      gtk_widget_measure (self->content, GTK_ORIENTATION_HORIZONTAL, for_size_min,
-                          &content_min, &content_nat, NULL, NULL);
-      if (for_size_nat != for_size_min) {
-        gtk_widget_measure (self->content, GTK_ORIENTATION_HORIZONTAL, for_size_nat,
-                            NULL, &content_nat, NULL, NULL);
-      }
-    }
-
-  }
+  if (self->content)
+    gtk_widget_measure (self->content, orientation, for_size,
+                        &content_min, &content_nat, NULL, NULL);
 
   if (orientation == GTK_ORIENTATION_HORIZONTAL) {
     if (minimum)
@@ -364,13 +333,13 @@ adw_toolbar_view_size_allocate (GtkWidget *widget,
   int top_height, bottom_height;
   int content_height, content_offset;
 
-  gtk_widget_measure (self->top_bar, GTK_ORIENTATION_VERTICAL, width,
+  gtk_widget_measure (self->top_bar, GTK_ORIENTATION_VERTICAL, -1,
                       &top_min, &top_nat, NULL, NULL);
-  gtk_widget_measure (self->bottom_bar, GTK_ORIENTATION_VERTICAL, width,
+  gtk_widget_measure (self->bottom_bar, GTK_ORIENTATION_VERTICAL, -1,
                       &bottom_min, &bottom_nat, NULL, NULL);
 
   if (self->content)
-    gtk_widget_measure (self->content, GTK_ORIENTATION_VERTICAL, width,
+    gtk_widget_measure (self->content, GTK_ORIENTATION_VERTICAL, -1,
                         &content_min, NULL, NULL, NULL);
 
   if (self->extend_content_to_top_edge)
@@ -524,7 +493,7 @@ adw_toolbar_view_class_init (AdwToolbarViewClass *klass)
   widget_class->compute_expand = adw_widget_compute_expand;
 
   /**
-   * AdwToolbarView:content:
+   * AdwToolbarView:content: (attributes org.gtk.Property.get=adw_toolbar_view_get_content org.gtk.Property.set=adw_toolbar_view_set_content)
    *
    * The content widget.
    *
@@ -536,13 +505,13 @@ adw_toolbar_view_class_init (AdwToolbarViewClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwToolbarView:top-bar-style:
+   * AdwToolbarView:top-bar-style: (attributes org.gtk.Property.get=adw_toolbar_view_get_top_bar_style org.gtk.Property.set=adw_toolbar_view_set_top_bar_style)
    *
    * Appearance of the top bars.
    *
    * If set to `ADW_TOOLBAR_FLAT`, top bars are flat and scrolling content has a
    * subtle undershoot shadow when touching them, same as the
-   * [`.undershoot-top`](style-classes.html#undershoot-indicators)
+   * [`.undershoot-top`](style-classes.html#undershot-indicators)
    * style class. This works well for simple content, e.g. [class@StatusPage] or
    * [class@PreferencesPage], where the background at the top of the page is
    * uniform. Additionally, windows with sidebars should always use this style.
@@ -572,13 +541,13 @@ adw_toolbar_view_class_init (AdwToolbarViewClass *klass)
                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwToolbarView:bottom-bar-style:
+   * AdwToolbarView:bottom-bar-style: (attributes org.gtk.Property.get=adw_toolbar_view_get_bottom_bar_style org.gtk.Property.set=adw_toolbar_view_set_bottom_bar_style)
    *
    * Appearance of the bottom bars.
    *
    * If set to `ADW_TOOLBAR_FLAT`, bottom bars are flat and scrolling content
    * has a subtle undershoot shadow when touching them, same as the
-   * [`.undershoot-bottom`](style-classes.html#undershoot-indicators)
+   * [`.undershoot-bottom`](style-classes.html#undershot-indicators)
    * style class. This works well for simple content, e.g. [class@StatusPage] or
    * [class@PreferencesPage], where the background at the bottom of the page is
    * uniform. Additionally, windows with sidebars should always use this style.
@@ -608,7 +577,7 @@ adw_toolbar_view_class_init (AdwToolbarViewClass *klass)
                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwToolbarView:reveal-top-bars:
+   * AdwToolbarView:reveal-top-bars: (attributes org.gtk.Property.get=adw_toolbar_view_get_reveal_top_bars org.gtk.Property.set=adw_toolbar_view_get_reveal_top_bars)
    *
    * Whether top bars are revealed.
    *
@@ -628,7 +597,7 @@ adw_toolbar_view_class_init (AdwToolbarViewClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwToolbarView:reveal-bottom-bars:
+   * AdwToolbarView:reveal-bottom-bars: (attributes org.gtk.Property.get=adw_toolbar_view_get_reveal_bottom_bars org.gtk.Property.set=adw_toolbar_view_get_reveal_bottom_bars)
    *
    * Whether bottom bars are visible.
    *
@@ -648,7 +617,7 @@ adw_toolbar_view_class_init (AdwToolbarViewClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwToolbarView:extend-content-to-top-edge:
+   * AdwToolbarView:extend-content-to-top-edge: (attributes org.gtk.Property.get=adw_toolbar_view_get_extend_content_to_top_edge org.gtk.Property.set=adw_toolbar_view_set_extend_content_to_top_edge)
    *
    * Whether the content widget can extend behind top bars.
    *
@@ -665,7 +634,7 @@ adw_toolbar_view_class_init (AdwToolbarViewClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwToolbarView:extend-content-to-bottom-edge:
+   * AdwToolbarView:extend-content-to-bottom-edge: (attributes org.gtk.Property.get=adw_toolbar_view_get_extend_content_to_bottom_edge org.gtk.Property.set=adw_toolbar_view_set_extend_content_to_bottom_edge)
    *
    * Whether the content widget can extend behind bottom bars.
    *
@@ -683,7 +652,7 @@ adw_toolbar_view_class_init (AdwToolbarViewClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwToolbarView:top-bar-height:
+   * AdwToolbarView:top-bar-height: (attributes org.gtk.Property.get=adw_toolbar_view_get_top_bar_height)
    *
    * The current top bar height.
    *
@@ -700,7 +669,7 @@ adw_toolbar_view_class_init (AdwToolbarViewClass *klass)
                       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * AdwToolbarView:bottom-bar-height:
+   * AdwToolbarView:bottom-bar-height: (attributes org.gtk.Property.get=adw_toolbar_view_get_bottom_bar_height)
    *
    * The current bottom bar height.
    *
@@ -803,7 +772,7 @@ adw_toolbar_view_new (void)
 }
 
 /**
- * adw_toolbar_view_get_content:
+ * adw_toolbar_view_get_content: (attributes org.gtk.Method.get_property=content)
  * @self: a toolbar view
  *
  * Gets the content widget for @self.
@@ -821,7 +790,7 @@ adw_toolbar_view_get_content (AdwToolbarView *self)
 }
 
 /**
- * adw_toolbar_view_set_content:
+ * adw_toolbar_view_set_content: (attributes org.gtk.Method.set_property=content)
  * @self: a toolbar view
  * @content: (nullable): the content widget
  *
@@ -836,12 +805,12 @@ adw_toolbar_view_set_content (AdwToolbarView *self,
   g_return_if_fail (ADW_IS_TOOLBAR_VIEW (self));
   g_return_if_fail (content == NULL || GTK_IS_WIDGET (content));
 
-  if (content == self->content)
-    return;
-
   if (content)
     g_return_if_fail (gtk_widget_get_parent (content) == NULL);
 
+  if (content == self->content)
+    return;
+
   if (self->content)
     gtk_widget_unparent (self->content);
 
@@ -944,7 +913,7 @@ adw_toolbar_view_remove (AdwToolbarView *self,
 }
 
 /**
- * adw_toolbar_view_get_top_bar_style:
+ * adw_toolbar_view_get_top_bar_style: (attributes org.gtk.Method.get_property=top-bar-style)
  * @self: a toolbar view
  *
  * Gets appearance of the top bars for @self.
@@ -962,7 +931,7 @@ adw_toolbar_view_get_top_bar_style (AdwToolbarView *self)
 }
 
 /**
- * adw_toolbar_view_set_top_bar_style:
+ * adw_toolbar_view_set_top_bar_style: (attributes org.gtk.Method.set_property=top-bar-style)
  * @self: a toolbar view
  * @style: top bar style
  *
@@ -970,7 +939,7 @@ adw_toolbar_view_get_top_bar_style (AdwToolbarView *self)
  *
  * If set to `ADW_TOOLBAR_FLAT`, top bars are flat and scrolling content has a
  * subtle undershoot shadow when touching them, same as the
- * [`.undershoot-top`](style-classes.html#undershoot-indicators)
+ * [`.undershoot-top`](style-classes.html#undershot-indicators)
  * style class. This works well for simple content, e.g. [class@StatusPage] or
  * [class@PreferencesPage], where the background at the top of the page is
  * uniform. Additionally, windows with sidebars should always use this style.
@@ -1030,10 +999,10 @@ adw_toolbar_view_set_top_bar_style (AdwToolbarView  *self,
 }
 
 /**
- * adw_toolbar_view_get_bottom_bar_style:
+ * adw_toolbar_view_get_bottom_bar_style: (attributes org.gtk.Method.get_property=bottom-bar-style)
  * @self: a toolbar view
  *
- * Gets appearance of the bottom bars for @self.
+ * Gets appearance of the botom bars for @self.
  *
  * Returns: bottom bar style
  *
@@ -1048,7 +1017,7 @@ adw_toolbar_view_get_bottom_bar_style (AdwToolbarView *self)
 }
 
 /**
- * adw_toolbar_view_set_bottom_bar_style:
+ * adw_toolbar_view_set_bottom_bar_style: (attributes org.gtk.Method.set_property=bottom-bar-style)
  * @self: a toolbar view
  * @style: bottom bar style
  *
@@ -1056,7 +1025,7 @@ adw_toolbar_view_get_bottom_bar_style (AdwToolbarView *self)
  *
  * If set to `ADW_TOOLBAR_FLAT`, bottom bars are flat and scrolling content has
  * a subtle undershoot shadow when touching them, same as the
- * [`.undershoot-bottom`](style-classes.html#undershoot-indicators)
+ * [`.undershoot-bottom`](style-classes.html#undershot-indicators)
  * style class. This works well for simple content, e.g. [class@StatusPage] or
  * [class@PreferencesPage], where the background at the bottom of the page is
  * uniform. Additionally, windows with sidebars should always use this style.
@@ -1116,7 +1085,7 @@ adw_toolbar_view_set_bottom_bar_style (AdwToolbarView  *self,
 }
 
 /**
- * adw_toolbar_view_get_reveal_top_bars:
+ * adw_toolbar_view_get_reveal_top_bars: (attributes org.gtk.Method.get_property=reveal-top-bars)
  * @self: a toolbar view
  *
  * Gets whether top bars are revealed for @self.
@@ -1134,7 +1103,7 @@ adw_toolbar_view_get_reveal_top_bars (AdwToolbarView *self)
 }
 
 /**
- * adw_toolbar_view_set_reveal_top_bars:
+ * adw_toolbar_view_set_reveal_top_bars: (attributes org.gtk.Method.set_property=reveal-top-bars)
  * @self: a toolbar view
  * @reveal: whether to reveal top bars
  *
@@ -1167,7 +1136,7 @@ adw_toolbar_view_set_reveal_top_bars (AdwToolbarView *self,
 }
 
 /**
- * adw_toolbar_view_get_reveal_bottom_bars:
+ * adw_toolbar_view_get_reveal_bottom_bars: (attributes org.gtk.Method.get_property=reveal-bottom-bars)
  * @self: a toolbar view
  *
  * Gets whether bottom bars are revealed for @self.
@@ -1185,7 +1154,7 @@ adw_toolbar_view_get_reveal_bottom_bars (AdwToolbarView *self)
 }
 
 /**
- * adw_toolbar_view_set_reveal_bottom_bars:
+ * adw_toolbar_view_set_reveal_bottom_bars: (attributes org.gtk.Method.set_property=reveal-bottom-bars)
  * @self: a toolbar view
  * @reveal: whether to reveal bottom bars
  *
@@ -1218,7 +1187,7 @@ adw_toolbar_view_set_reveal_bottom_bars (AdwToolbarView *self,
 }
 
 /**
- * adw_toolbar_view_get_extend_content_to_top_edge:
+ * adw_toolbar_view_get_extend_content_to_top_edge: (attributes org.gtk.Method.get_property=extend-content-to-top-edge)
  * @self: a toolbar view
  *
  * Gets whether the content widget can extend behind top bars.
@@ -1236,7 +1205,7 @@ adw_toolbar_view_get_extend_content_to_top_edge (AdwToolbarView *self)
 }
 
 /**
- * adw_toolbar_view_set_extend_content_to_top_edge:
+ * adw_toolbar_view_set_extend_content_to_top_edge: (attributes org.gtk.Method.set_property=extend-content-to-top-edge)
  * @self: a toolbar view
  * @extend: whether content extends behind top bars
  *
@@ -1270,7 +1239,7 @@ adw_toolbar_view_set_extend_content_to_top_edge (AdwToolbarView *self,
 }
 
 /**
- * adw_toolbar_view_get_extend_content_to_bottom_edge:
+ * adw_toolbar_view_get_extend_content_to_bottom_edge: (attributes org.gtk.Method.get_property=extend-content-to-bottom-edge)
  * @self: a toolbar view
  *
  * Gets whether the content widget can extend behind bottom bars.
@@ -1288,7 +1257,7 @@ adw_toolbar_view_get_extend_content_to_bottom_edge (AdwToolbarView *self)
 }
 
 /**
- * adw_toolbar_view_set_extend_content_to_bottom_edge:
+ * adw_toolbar_view_set_extend_content_to_bottom_edge: (attributes org.gtk.Method.set_property=extend-content-to-bottom-edge)
  * @self: a toolbar view
  * @extend: whether content extends behind bottom bars
  *
@@ -1322,7 +1291,7 @@ adw_toolbar_view_set_extend_content_to_bottom_edge (AdwToolbarView *self,
 }
 
 /**
- * adw_toolbar_view_get_top_bar_height:
+ * adw_toolbar_view_get_top_bar_height: (attributes org.gtk.Method.get_property=top-bar-height)
  * @self: a toolbar view
  *
  * Gets the current top bar height for @self.
@@ -1345,7 +1314,7 @@ adw_toolbar_view_get_top_bar_height (AdwToolbarView *self)
 }
 
 /**
- * adw_toolbar_view_get_bottom_bar_height:
+ * adw_toolbar_view_get_bottom_bar_height: (attributes org.gtk.Method.get_property=bottom-bar-height)
  * @self: a toolbar view
  *
  * Gets the current bottom bar height for @self.
