@@ -15,7 +15,6 @@
 #include <gtk/gtk.h>
 
 #include "adw-breakpoint.h"
-#include "adw-dialog.h"
 
 G_BEGIN_DECLS
 
@@ -48,16 +47,4 @@ void adw_window_add_breakpoint (AdwWindow     *self,
 ADW_AVAILABLE_IN_1_4
 AdwBreakpoint *adw_window_get_current_breakpoint (AdwWindow *self);
 
-ADW_AVAILABLE_IN_1_5
-GListModel *adw_window_get_dialogs (AdwWindow *self);
-
-ADW_AVAILABLE_IN_1_5
-AdwDialog *adw_window_get_visible_dialog (AdwWindow *self);
-
-ADW_AVAILABLE_IN_1_7
-gboolean adw_window_get_adaptive_preview (AdwWindow *self);
-ADW_AVAILABLE_IN_1_7
-void     adw_window_set_adaptive_preview (AdwWindow *self,
-                                          gboolean   adaptive_preview);
-
 G_END_DECLS
